Package: BIGr
Title: Breeding Insight Genomics Functions for Polyploid and Diploid
        Species
Version: 0.7.2
Authors@R: c(person(given='Alexander M.',
                    family='Sandercock',
                    email='sandercock.alex@gmail.com',
                    role=c('cre','aut')),
             person(given='Cristiane',
                    family='Taniguti',
                    role = 'aut'),
             person(given='Josue',
                    family='Chinchilla-Vargas',
                    role='aut'),
             person(given='Shufen',
                    family='Chen',
                    role='ctb'),
             person(given='Manoj',
                    family='Sapkota',
                    role='ctb'),
             person(given='Meng',
                    family='Lin',
                    role='ctb'),
             person(given='Dongyan',
                    family='Zhao',
                    role='ctb'),
             person('University', "of Florida",
                    role=c('cph'), 
                    comment = "Breeding Insight"))
Maintainer: Alexander M. Sandercock <sandercock.alex@gmail.com>
Description: Functions developed within Breeding Insight to analyze
    diploid and polyploid breeding and genetic data. 'BIGr' provides the
    ability to filter variant call format (VCF) files, extract single nucleotide polymorphisms (SNPs) 
    from diversity arrays technology missing allele discovery count (DArT MADC) files, 
    and manipulate genotype data for both diploid and polyploid species. It
    also serves as the core dependency for the 'BIGapp' 'Shiny' app, which
    provides a user-friendly interface for performing routine genotype
    analysis tasks such as dosage calling, filtering, principal component analysis (PCA), 
    genome-wide association studies (GWAS), and
    genomic prediction. For more details about the included 'breedTools' 
    functions, see Funkhouser et al. (2017) <doi:10.2527/tas2016.0003>, and
    the 'updog' output format, see Gerard et al. (2018) <doi:10.1534/genetics.118.301468>.
License: Apache License (>= 2)
URL: https://github.com/Breeding-Insight/BIGr
BugReports: https://github.com/Breeding-Insight/BIGr/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.4.0)
biocViews:
Imports: parallel, dplyr, Rdpack (>= 0.7), readr (>= 2.1.5), reshape2
        (>= 1.4.4), rlang, tidyr (>= 1.3.1), vcfR (>= 1.15.0),
        Rsamtools, Biostrings, pwalign, janitor, quadprog, tibble,
        stringr, data.table
Suggests: covr, ggplot2, spelling, rmdformats, knitr (>= 1.10),
        rmarkdown, polyRAD, testthat (>= 3.0.0)
RdMacros: Rdpack
NeedsCompilation: no
Packaged: 2026-05-17 23:05:43 UTC; aherranssanderco
Author: Alexander M. Sandercock [cre, aut],
  Cristiane Taniguti [aut],
  Josue Chinchilla-Vargas [aut],
  Shufen Chen [ctb],
  Manoj Sapkota [ctb],
  Meng Lin [ctb],
  Dongyan Zhao [ctb],
  University of Florida [cph] (Breeding Insight)
Repository: CRAN
Date/Publication: 2026-05-17 23:30:02 UTC
Built: R 4.5.2; ; 2026-05-18 00:44:14 UTC; unix
